@@ -9,12 +9,14 @@ import json
 import uuid
 import gc
 from typing import Dict, List, Optional
+from uuid import UUID
 
 import numpy as np
 import requests
 from loguru import logger
 from pydantic import BaseModel, Field
 from tqdm import tqdm
+from datetime import date
 
 from .utils import get_random_name
 from .cli import get_api_credentials, refresh_bearer_token, validate_api_http_response
@@ -53,6 +55,12 @@ def get_object_size_in_bytes(obj):
 
     return sz
 
+def assert_valid_project_id(project_id):
+    try:
+        uuid_obj = UUID(project_id, version=4)
+    except ValueError:
+        raise ValueError(f"`{project_id}` is not a valid project id.")
+
 class CreateIndexResponse(BaseModel):
     map: Optional[str] = Field(
         None, description="A link to the map this index creates. May take some time to be ready so check the job state."
@@ -242,6 +250,8 @@ class AtlasClient:
             Returns the requested project.
         '''
 
+        assert_valid_project_id(project_id)
+
         response = requests.get(
             self.atlas_api_path + f"/v1/project/{project_id}",
             headers=self.header,
@@ -275,8 +285,12 @@ class AtlasClient:
     def _validate_and_correct_user_supplied_metadata(self, data: List[Dict], project, replace_empty_string_values_with_string_null=True):
         '''
         Validates the users metadata for Atlas compatability.
-        If unique_id_field is specified, validates that each datum has that field. If not, adds it
-        and then notifies the user that it was added.
+
+        1. If unique_id_field is specified, validates that each datum has that field. If not, adds it and then notifies the user that it was added.
+
+        2. If a key is detected to store values that match an ISO8601 timestamp string ,Atlas will assume you are working with timestamps. If any additional metadata
+        has this key associated with a non-ISO8601 timestamp string the upload will fail.
+
         Args:
             data: the user supplied list of data dictionaries
             project: the atlas project you are validating the data for.
@@ -285,19 +299,23 @@ class AtlasClient:
         Returns:
 
         '''
-
         if not isinstance(data, list):
             raise Exception("Metadata must be a list of dictionaries")
 
         metadata_keys = None
-        for datum in data:
+        metadata_date_keys = []
 
+        for datum in data:
             #The Atlas client adds a unique datum id field for each user.
             #It does not overwrite the field if it exists, instead map creation fails.
             if project['unique_id_field'] in datum:
-                raise Exception(f"`{project['unique_id_field']}` is an invalid field type. Atlas uses the `{project['unique_id_field']}` field to uniquely identify datums.")
+                if len(str(datum[project['unique_id_field']])) > 36:
+                    raise ValueError(f"{datum}\n The id_field `{datum[project['unique_id_field']]}` is greater than 36 characters. Atlas does not support id_fields longer than 36 characters.")
             else:
-                datum[project['unique_id_field']] = str(uuid.uuid4())
+                if project['unique_id_field'] == ATLAS_DEFAULT_ID_FIELD:
+                    datum[project['unique_id_field']] = str(uuid.uuid4())
+                else:
+                    raise ValueError(f"{datum} does not contain your specified id_field `{datum[project['unique_id_field']]}`")
 
             if not isinstance(datum, dict):
                 raise Exception('Each metadata must be a dictionary with one level of keys and values of only string, int and float types.')
@@ -305,6 +323,14 @@ class AtlasClient:
             if metadata_keys is None:
                 metadata_keys = sorted(list(datum.keys()))
 
+                #figure out which are dates
+                for key in metadata_keys:
+                    try:
+                        date.fromisoformat(str(datum[key]))
+                        metadata_date_keys.append(key)
+                    except ValueError:
+                        pass
+
             datum_keylist = sorted(list(datum.keys()))
             if datum_keylist != metadata_keys:
                 msg = 'All metadata must have the same keys, but found key sets: {} and {}'.format(metadata_keys, datum_keylist)
@@ -314,6 +340,13 @@ class AtlasClient:
                 if key.startswith('_'):
                     raise ValueError('Metadata fields cannot start with _')
 
+                if key in metadata_date_keys:
+                    try:
+                        date.fromisoformat(str(datum[key]))
+                    except ValueError:
+                        raise ValueError(f"{datum} has timestamp key `{key}` which cannot be parsed as a ISO8601 string. See the following documentation in the Nomic client for working with timestamps: https://docs.nomic.ai/mapping_faq.html.")
+
+
                 if project['modality'] == 'text':
                     if isinstance(datum[key], str) and len(datum[key]) == 0:
                         if replace_empty_string_values_with_string_null:
@@ -337,6 +370,8 @@ class AtlasClient:
 
         **Returns:** True if project is unlocked for data additions, false otherwise.
         '''
+        assert_valid_project_id(project_id)
+
         response = requests.get(
             self.atlas_api_path+ f"/v1/project/{project_id}",
             headers=self.header,
@@ -363,6 +398,7 @@ class AtlasClient:
 
         **Returns:** True on success.
         '''
+        assert_valid_project_id(project_id)
 
         # Each worker currently is to slow beyond a shard_size of 5000
         shard_size = min(shard_size, 5000)
@@ -465,7 +501,7 @@ class AtlasClient:
 
         **Returns:** A link to your map.
         '''
-
+        assert_valid_project_id(project_id)
 
         project = self.get_project_by_id(project_id=project_id)
 
@@ -493,6 +529,8 @@ class AtlasClient:
                 headers=self.header,
                 json=embedding_build_template,
             )
+
+            print(response.json())
             job_id = response.json()['job_id']
 
         elif project['modality'] == 'text':
@@ -581,6 +619,7 @@ class AtlasClient:
 
         **Returns:** True on success.
         '''
+        assert_valid_project_id(project_id)
 
 
         # Each worker currently is to slow beyond a shard_size of 5000
@@ -668,6 +707,7 @@ class AtlasClient:
         self,
         embeddings: np.array,
         data: List[Dict] = None,
+        id_field: str = None,
         is_public: bool = True,
         colorable_fields: list = [],
         num_workers: int = 10,
@@ -686,6 +726,7 @@ class AtlasClient:
         * **embeddings** - An [N,d] numpy array containing the batch of N embeddings to add.
         * **data** - An [N,] element list of dictionaries containing metadata for each embedding.
         * **colorable_fields** - The project fields you want to be able to color by on the map. Must be a subset of the projects fields.
+        * **id_field** - Specify your datas unique id field. ID fields can be up 36 characters in length. If not specified, one will be created for you named `id_`.
         * **is_public** - Should this embedding map be public? Private maps can only be accessed by members of your organization.
         * **num_workers** - The number of workers to use when sending data.
         * **map_name** - A name for your map.
@@ -697,7 +738,8 @@ class AtlasClient:
 
         **Returns:** A link to your map.
         '''
-        id_field = ATLAS_DEFAULT_ID_FIELD # do not let user specify a unique id field, handle it for them.
+        if id_field is None:
+            id_field = ATLAS_DEFAULT_ID_FIELD # do not let user specify a unique id field, handle it for them.
 
         project_name = get_random_name()
         description = project_name
@@ -844,6 +886,7 @@ class AtlasClient:
         self,
         data: List[Dict],
         indexed_field: str,
+        id_field: str = None,
         is_public: bool = True,
         colorable_fields: list = [],
         num_workers: int = 10,
@@ -861,6 +904,7 @@ class AtlasClient:
 
         * **data** - An [N,] element list of dictionaries containing metadata for each embedding.
         * **indexed_field** - The name the data field corresponding to the text to be mapped.
+        * **id_field** - Specify your datas unique id field. ID fields can be up 36 characters in length. If not specified, one will be created for you named `id_`.
         * **colorable_fields** - The project fields you want to be able to color by on the map. Must be a subset of the projects fields.
         * **is_public** - Should this embedding map be public? Private maps can only be accessed by members of your organization.
         * **num_workers** - The number of workers to use when sending data.
@@ -873,7 +917,9 @@ class AtlasClient:
 
         **Returns:** A link to your map.
         '''
-        id_field = ATLAS_DEFAULT_ID_FIELD
+        if id_field is None:
+            id_field = ATLAS_DEFAULT_ID_FIELD
+
         project_name = get_random_name()
         description = project_name
         index_name = get_random_name()
@@ -980,7 +1026,7 @@ class AtlasClient:
             )
 
             if response.status_code != 200:
-                raise Exception(response.json()['detail'])
+                raise Exception(response.json())
 
             if check_access:
                 return
