@@ -1,6 +1,7 @@
 import uuid
 import time
 import tempfile
+import datetime
 
 from nomic import AtlasClient
 import pytest
@@ -29,13 +30,20 @@ def test_map_embeddings_with_errors():
         response = atlas.map_embeddings(embeddings=embeddings,
                                         data=data,
                                         map_name='UNITTEST',
-                                        id_field='id',
                                         is_public=True)
 
     #test non-matching keys across metadatums
     with pytest.raises(Exception):
         data = [{'hello': 'a'} for i in range(len(embeddings))]
         data[1]['goodbye'] = 'b'
+        response = atlas.map_embeddings(embeddings=embeddings,
+                                        data=data,
+                                        map_name='UNITTEST',
+                                        is_public=True)
+
+    #test to long ids
+    with pytest.raises(Exception):
+        data = [{'id': str(uuid.uuid4())+'a'} for i in range(len(embeddings))]
         response = atlas.map_embeddings(embeddings=embeddings,
                                         data=data,
                                         map_name='UNITTEST',
@@ -66,10 +74,11 @@ def test_map_embeddings():
 
     num_embeddings = 10
     embeddings = np.random.rand(num_embeddings, 10)
-    data = [{'field': str(uuid.uuid4())} for i in range(len(embeddings))]
+    data = [{'field': str(uuid.uuid4()), 'id': str(uuid.uuid4())} for i in range(len(embeddings))]
 
     response = atlas.map_embeddings(embeddings=embeddings,
                                     map_name='UNITTEST',
+                                    id_field='id',
                                     data=data,
                                     is_public=True)
 
@@ -77,7 +86,7 @@ def test_map_embeddings():
     project_id = response['project_id']
     project = atlas.get_project_by_id(project_id)
     atlas_index_id = project['atlas_indices'][0]['id']
-    data = [{'field': str(uuid.uuid4())} for i in range(len(embeddings))]
+    data = [{'field': str(uuid.uuid4()), 'id': str(uuid.uuid4())} for i in range(len(embeddings))]
     while True:
         # Code executed here
         time.sleep(10)
@@ -92,6 +101,34 @@ def test_map_embeddings():
     atlas.delete_project(project_id=response['project_id'])
 
 
+def test_date_metadata():
+    atlas = AtlasClient()
+
+    num_embeddings = 10
+    embeddings = np.random.rand(num_embeddings, 10)
+    data = [{'my_date': datetime.datetime(2022, 1, i).isoformat()} for i in range(1, len(embeddings)+1)]
+
+    response = atlas.map_embeddings(embeddings=embeddings,
+                                    map_name='UNITTEST',
+                                    data=data,
+                                    is_public=True)
+
+    assert response['project_id']
+
+    atlas.delete_project(project_id=response['project_id'])
+
+
+    #put an invalid iso timestamp after the first valid isotimestamp , make sure the client fails
+    with pytest.raises(Exception):
+        data[1]['my_date'] = data[1]['my_date']+'asdf'
+        response = atlas.map_embeddings(embeddings=embeddings,
+                                        map_name='UNITTEST',
+                                        id_field='id',
+                                        data=data,
+                                        is_public=True)
+
+
+
 def test_map_text_errors():
     atlas = AtlasClient()
 
